from pydrinker.routes import DrinkerRoute

from .message_translators import SubscriptionMessageTranslator
from .providers import SubscriptionProvider


class SubscriptionRoute(DrinkerRoute):
    def __init__(
        self,
        project_id,
        subscription_id,
        provider_options=None,
        message_translator=None,
        name=None,
        *args,
        **kwargs,
    ):
        provider_options = provider_options or {}
        provider = SubscriptionProvider(
            project_id=project_id, subscription_id=subscription_id, **provider_options
        )
        kwargs["provider"] = provider
        kwargs["message_translator"] = message_translator or SubscriptionMessageTranslator()
        kwargs["name"] = name or f"{project_id}/{subscription_id}"

        super().__init__(*args, **kwargs)



import json
import logging

from google.cloud.pubsub_v1.types import ReceivedMessage
from pydrinker.message_translators import AbstractMessageTranslator

logger = logging.getLogger(__name__)


class SubscriptionMessageTranslator(AbstractMessageTranslator):
    def translate(self, message: ReceivedMessage):
        """Translate a given message to an appropriate format to message processing.

        This method should return a `dict` instance with two keys: `content`
        and `metadata`.
        The `content` should contain the translated message and, `metadata` a
        dictionary with translation metadata.
        """
        translated_message = {"content": None}
        pubsub_message = message.message
        translated_message["metadata"] = {
            "ack_id": message.ack_id,
            "message_id": pubsub_message.message_id,
            "publish_time": pubsub_message.publish_time,
            "ordering_key": pubsub_message.ordering_key,
            "attributes": pubsub_message.attributes,
        }

        try:
            decoded_message_data = pubsub_message.data.decode("utf-8")
        except UnicodeDecodeError as exc:
            logger.error(f"error={exc!r}, message={message!r}")
            return translated_message

        try:
            translated_message["content"] = json.loads(decoded_message_data)
        except json.decoder.JSONDecodeError as exc:
            logger.error(f"error={exc!r}, message={message!r}")
            return translated_message

        return translated_message



import json
import os

from google.api_core import retry
from google.auth import jwt
from google.cloud import pubsub_v1
from pydrinker.exceptions import ProviderError

SUB_AUDIENCE = "https://pubsub.googleapis.com/google.pubsub.v1.Subscriber"


def _get_subscriber():
    credential_file = os.environ.get("GOOGLE_APPLICATION_CREDENTIALS")
    if credential_file and os.path.isfile(credential_file):
        return pubsub_v1.SubscriberClient()

    google_service_account = os.environ.get("GOOGLE_SERVICE_ACCOUNT")
    if google_service_account:
        if isinstance(google_service_account, str):
            google_service_account = json.loads(google_service_account)
        credentials = jwt.Credentials.from_service_account_info(
            google_service_account,
            audience=SUB_AUDIENCE,
        )
        return pubsub_v1.SubscriberClient(credentials=credentials)

    raise ProviderError(
        "cannot set subscriber without GOOGLE_APPLICATION_CREDENTIALS or "
        "GOOGLE_SERVICE_ACCOUNT environment variables"
    )


class BaseSubscriber:
    def __init__(self, project_id: str, subscription_id: str, *args, **kwargs):
        self.subscriber = _get_subscriber()
        self.subscription_path = self.subscriber.subscription_path(project_id, subscription_id)

    def get_messages(
        self, deadline: float = 300, max_messages: int = 1, timeout: float = None, *args, **kwargs
    ):
        response = self.subscriber.pull(
            request={"subscription": self.subscription_path, "max_messages": max_messages},
            retry=retry.Retry(deadline=deadline),
            timeout=timeout,
        )
        for received_message in response.received_messages:
            yield received_message

    def acknowledge_messages(
        self, ack_ids: list, deadline: float = 300, timeout: float = None, *args, **kwargs
    ) -> None:
        self.subscriber.acknowledge(
            request={"subscription": self.subscription_path, "ack_ids": ack_ids},
            retry=retry.Retry(deadline=deadline),
            timeout=timeout,
        )

    def close(self):
        self.subscriber.close()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pydrinker_gcp']

package_data = \
{'': ['*']}

install_requires = \
['google-cloud-pubsub>=2.8.0,<3.0.0', 'pydrinker>=1.0.0,<2.0.0']

setup_kwargs = {
    'name': 'pydrinker-gcp',
    'version': '1.1.2',
    'description': "Google Cloud Platform 'plugin' for pydrinker",
    'long_description': '# pydrinker-gcp\n![build on github actions](https://github.com/pydrinker/pydrinker-gcp/actions/workflows/test.yml/badge.svg?branch=main)\n\nThis is a extension of pydrinker for Google Cloud Provider to make pydrinker consume messages from [GCP Subscribers](https://cloud.google.com/pubsub/docs/subscriber).\n\nTo understand more about pydrinker [see org page](https://github.com/pydrinker).\n',
    'author': 'Rafael Henrique da Silva Correia',
    'author_email': 'rafael@abraseucodigo.com.br',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/rafaelhenrique/pydrinker-gcp',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<3.10',
}


setup(**setup_kwargs)



import logging

from google import api_core
from pydrinker.exceptions import ProviderError
from pydrinker.providers import AbstractProvider

from .base import BaseSubscriber

logger = logging.getLogger(__name__)

GOOGLE_CORE_EXCEPTIONS = (
    api_core.exceptions.Aborted,
    api_core.exceptions.DeadlineExceeded,
    api_core.exceptions.InternalServerError,
    api_core.exceptions.ResourceExhausted,
    api_core.exceptions.ServiceUnavailable,
    api_core.exceptions.Unknown,
    api_core.exceptions.Cancelled,
)


class SubscriptionProvider(AbstractProvider, BaseSubscriber):
    def __init__(self, project_id: str, subscription_id: str, options=None, **kwargs):
        self.project_id = project_id
        self.subscription_id = subscription_id
        self._options = options or {}
        super().__init__(project_id, subscription_id, **kwargs)

    async def fetch_messages(self):
        """Return a sequence of messages to be processed.

        If no messages are available, this coroutine should return an empty list.
        """
        logger.debug(f"fetching messages on {self.subscription_id}")
        try:
            messages = list(self.get_messages(**self._options))
        except GOOGLE_CORE_EXCEPTIONS as exc:
            raise ProviderError(
                f"error to fetch messages from subscriber_id={self.subscription_id!r}: {exc}"
            ) from exc

        return messages or []

    async def confirm_message(self, message):
        """Confirm the message processing.

        After the message confirmation we should not receive the same message again.
        This usually means we need to delete/acknowledge the message in the provider.
        """
        ack_id = message.ack_id
        logger.info(f"confirm message (ack/deletion), ack_id={ack_id}")
        try:
            self.acknowledge_messages(ack_ids=[ack_id], **self._options)
        except GOOGLE_CORE_EXCEPTIONS as exc:
            raise ProviderError(
                f"error to confirm messages from subscriber_id={self.subscription_id!r}: {exc}"
            ) from exc

    def stop(self):
        """Stop the provider.

        If needed, the provider should perform clean-up actions.
        This method is called whenever we need to shutdown the provider.
        """
        logger.info(f"stopping {self}")
        self.close()
        return super().stop()
